@@ -188,28 +188,28 @@ static size_t		bytesPerPage = 0;
  /*
  * mutex to enable multithreaded operation
  */
-static pthread_mutex_t mutex ;
-static pid_t mutexpid=0;
+static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_t mutexowner = (pthread_t) -1;
 static int locknr=0;
 
 
 static void lock() {
     if (pthread_mutex_trylock(&mutex)) {
-       if (mutexpid==getpid()) {
+       if (mutexowner==pthread_self()) {
            locknr++;
            return;
        } else {
            pthread_mutex_lock(&mutex);
        }
     } 
-    mutexpid=getpid();
+    mutexowner=pthread_self();
     locknr=1;
 }
 
 static void unlock() {
     locknr--;
     if (!locknr) {
-       mutexpid=0;
+       mutexowner=(pthread_t)-1;
        pthread_mutex_unlock(&mutex);
     }
 }
@@ -429,6 +429,8 @@ memalign(size_t alignment, size_t userSize)
 	size_t		slack;
 	char *		address;
 
+	lock();
+
 	if ( allocationList == 0 )
 		initialize();
 
@@ -615,6 +617,7 @@ memalign(size_t alignment, size_t userSize)
 	if ( !internalUse )
 		Page_DenyAccess(allocationList, allocationListSize);
 
+	unlock();
 	return address;
 }
 
@@ -814,11 +817,9 @@ malloc(size_t size)
 {
         void  *allocation;   
  
-        if ( allocationList == 0 ) {
-                pthread_mutex_init(&mutex, NULL); 
-                initialize();   /* This sets EF_ALIGNMENT */
-        }       
         lock();
+        if ( allocationList == 0 )
+                initialize();   /* This sets EF_ALIGNMENT */
         allocation=memalign(EF_ALIGNMENT, size); 
 
         unlock();
@@ -833,10 +834,9 @@ calloc(size_t nelem, size_t elsize)
         void * allocation;
         
         lock();
-       
         allocation = malloc(size);
-        memset(allocation, 0, size);
         unlock();
+        memset(allocation, 0, size);
 
 	return allocation;
 }
